// ======================================================================
//
// MenuStaticListBox.h
// Jason Hughes
//
// copyright 1998 origin systems inc
//
// ======================================================================

#ifndef MENU_STATIC_LIST_BOX_H
#define MENU_STATIC_LIST_BOX_H

// ======================================================================

class String;

#include "Font.h"
#include "MenuElement.h"

// ======================================================================

class MenuStaticListBox : public MenuElement
{
private:

	struct Link
	{
		String *string;
		Font   *overridingFont; // if it's NOT NULL this will override the default font (NOT the highlighted value)
		Link   *prev;
		Link   *next;

		Link(void);
		~Link(void);

		void setString(const String &str, Font *overrideFont = NULL);
	};

protected:

	int  visibleLinks;

	int  *stringSelectEvent;
	int  numStringEvents;

	Font *font;
	Font *fontHighlight;
	Link *firstLink;
	Link *lastLink;
	Link *topLink;
	Link *selectedLink;
	int  curNumStrings;
	int  textHeight;    // Rob was asking for a way to do this - this will default to font->yCell() if not specified
	Font::JustificationRule justificationRule;

private:

	MenuStaticListBox(void);
	MenuStaticListBox(const MenuStaticListBox &);
	MenuStaticListBox &operator =(const MenuStaticListBox &);

	void             init(void);
	void             addLinkToEnd(Link *link);
	void             realAddString(const String &string, Font *overrideFont = NULL);

public:

	MenuStaticListBox(MenuElement *newParent, int maxWidth, int maxHeight, Iff *iff=NULL);
	virtual ~MenuStaticListBox(void);

	void             load(Iff *iff);

	virtual IoResult event(IoEventStruct *event);
	virtual void     menuDraw(int x, int y) const;

	void             addString(const String &string, Font *overrideFont = NULL);
	void             addString(int intToAdd, Font *overrideFont = NULL);
	void             scrollUp(void);
	void             scrollDown(void);
	void             scrollAbsolute(real percentage);

	int              getVisibleEntries(void) const;
	real             getPercentage(void) const;

	bool             stringExists(const String &string) const;

	void             removeAllStrings(void);
	void             removeString(const String &string);

	void             changeCurrentSelectedString(const String &newString, Font *overrideFont=NULL);
	void             changeString(const String &oldString, const String &newString, Font *overrideFont=NULL);
	const String    &getString(int absolutePosition) const;
	int              index(const String &string) const;
	int              getCurrentSelectedStringOffset(void) const;

	void             resetCurrentSelectedString(void);
	void             setCurrentSelectedString(int absoluteIndex);
	void             setCurrentSelectedString(const String &string);
	const String    *getCurrentSelectedString(void) const;

	int              getCurrentNumberOfStrings(void) const;

	void             setFont(Font *newFont);
	const Font       *getFont(void) const;
};

// ======================================================================

inline int MenuStaticListBox::getVisibleEntries(void) const
{
	return visibleLinks;
}

// ----------------------------------------------------------------------

inline int MenuStaticListBox::getCurrentNumberOfStrings(void) const
{
	return curNumStrings;
}

// ======================================================================

#endif
